import { useEffect, useMemo, useRef, useState } from "react";
import { Graph } from "@antv/x6";
import { History } from "@antv/x6-plugin-history";
import { Transform } from "@antv/x6-plugin-transform";
import { Scroller } from "@antv/x6-plugin-scroller";
import "@/components/TableNode";
import { GetAllDesignTables } from "@/api";
import { useRequest } from "umi";
export default function erModel() {
  const graphRef = useRef<Graph>();
  const [graph, setGraph] = useState<Graph>();

  const initGraph = (container: HTMLElement) => {
    const instance = new Graph({
      container,
      width: document.documentElement.clientWidth,
      height: document.documentElement.clientHeight,
      autoResize: true,
      async: false,
      mousewheel: {
        enabled: true,
        modifiers: "ctrl",
        minScale: 0.2,
        maxScale: 2,
      },
      connecting: {
        connectionPoint: "anchor",
      },
      grid: {
        visible: false,
        size: 5
      },
      background: {
        color: "#F2F7FA",
      },
      interacting: {
        nodeMovable: (view) => {
          const data = view.cell.getData<{
            ignoreDrag: boolean;
            lock: boolean;
          }>();
          // 禁止拖拽或锁节点
          if (data?.ignoreDrag || data?.lock) return false;
          return true;
        },
      },
    });

    instance.use(new History());
    instance.use(new Transform({
      resizing: {
        enabled: true,
      }
    }));
    instance.use(new Scroller());

    setGraph(instance);
    graphRef.current = instance;
  };

  const { data, run, loading } = useRequest(() => GetAllDesignTables({ groupType: "" }), {
    manual: false
  });

  useEffect(() => {
    const { result } = data || {};
    if(result && graphRef.current) {
      const { lines, objects, points, tableColumnList, tableList } = result;
      // 创建表格节点
      tableList?.forEach((table) => {
        graphRef.current?.addNode({
          shape: "table-node",
          x: 300,
          y: 100,
          width: 200,
          height: 200,
          data: {
            table,
            tableColumnList: tableColumnList?.filter(item => item.businessTableId === table.id)
          }
        })
      });
    }
  }, [graphRef.current, data]);

  return {
    initGraph,
    graph,
    graphRef,
    loading
  };
}
